#include <Arduboy2.h>
Arduboy2 arduboy;


#define LINKS             0
#define RECHTS            1
#define RAUF              2
#define RUNTER            3
#define KOLLISION         4
#define KEINE_KOLLISION   5
#define KEINE_KONTROLLE   6

#define ZEICHNUNG(n)  const unsigned char n[] PROGMEM   


int Ziel_stab_laenge = 15;
int Ziel_stab_position_y;
int Ziel_stab_position_x;

int Display_breite = 128;
int Display_hoehe  = 64;

int Ball_breite = 16;
int Ball_hoehe  = 16;


// **********************************************
// Speicher im Mikrokontroller
// **********************************************
ZEICHNUNG(Ball) = {
0xe0, 0x10, 0x0c, 0x04, 0x02, 0x01, 0x01, 0x01, 0x01, 0x01, 0x01, 0x02, 0x04, 0xc, 0x30, 0xc0, 0x07, 0x08, 0x30, 0x20, 0x40, 0x40, 0x80, 0x80, 0x80, 0x80, 0x40, 0x40, 0x20, 0x30, 0x0c, 0x03
};

ZEICHNUNG(BallX) = {
0x0f, 0x1f, 0x3f, 0x7f, 0xfe, 0xfc, 0xf8, 0xf0, 0xf0, 0xf8, 0xfc, 0xfe, 0x7f, 0x3f, 0x1f, 0x0f, 0xf0, 0xf8, 0xfc, 0xfe, 0x7f, 0x3f, 0x1f, 0x0f, 0x0f, 0x1f, 0x3f, 0x7f, 0xfe, 0xfc, 0xf8, 0xf0
};

// Punkte
int punkte;

// Richtung vom Ball
int richtung;
int richtung_saegezahn;

// Position vom Ball
int bx;
int by;
int kollision;

char speicher[100];

// **********************************************
// Start des Programmes.
// Wird nur einmal abgearbeitet.
// **********************************************
void setup()
{
  // Richtung vom Ball
  richtung = RECHTS;
  richtung_saegezahn = RUNTER;
  
  // Position vom Ball
  bx = 0;
  by = 20;

  kollision = KEINE_KONTROLLE;

  Ziel_stab_position_y = 25;
  Ziel_stab_position_x = 0;

  punkte = 0;

  arduboy.begin();
  arduboy.clear();
}


// **********************************************
// Unsere Endlosschleife START
// **********************************************
void loop()
{
  if ( arduboy.nextFrame() )
  {
    // **********************************************
    // Das Display löschen - Alles Schwarz
    // **********************************************
    arduboy.clear();

    // **********************************************
    // Regeln
    // **********************************************
    Ziel_stab_position_y = Joystick_abfragen_rauf_runter(Ziel_stab_position_y);
    Ziel_stab_position_x = Joystick_abfragen_rechts_links(Ziel_stab_position_x);
    by = Saegezahn(by);
    bx = Ball_Regeln(bx);
    kollision = Kollissionsdetektierung(bx, by, Ziel_stab_position_y, Ziel_stab_position_x, Ziel_stab_laenge);

    // **********************************************
    // Unseren Ball im Speicher Zeichnen
    // **********************************************
    if (kollision == KEINE_KONTROLLE)
    {
      arduboy.drawBitmap(bx, by, Ball, Ball_breite, Ball_hoehe, WHITE);
    }
    
    if (kollision == KOLLISION)
    {
      arduboy.drawBitmap(bx, by, BallX, Ball_breite, Ball_hoehe, WHITE);
    }
  
    // **********************************************
    // Unser Ziel im Speicher Zeichnen
    // **********************************************
    arduboy.drawLine(Ziel_stab_position_x, Ziel_stab_position_y, Ziel_stab_position_x, Ziel_stab_position_y + Ziel_stab_laenge, WHITE);

    // **********************************************
    // Text Zeichnen
    // **********************************************
    arduboy.setCursor(37, 0);
    sprintf (speicher, "Punkte: %d", punkte);
    arduboy.print(speicher);

    // **********************************************
    // Nun das gezeichnete auf's Display kopieren.
    // **********************************************
    arduboy.display();

    if (kollision == KOLLISION)
    {
      delay(50);
      punkte = punkte + 1;
    }
   
  }
}
// **********************************************
// Unsere Endlosschleife ENDE
// **********************************************


// **********************************************
// Das Zeichnen des Ball's mit den Regeln - START
// **********************************************
int Ball_Regeln (int x)
{
  // **********************************************
  // Unsere Regeln
  // **********************************************

  // Ball fliegt nach rechts
  if ( richtung == RECHTS )
  {
    // Wurde der rechte Rand erreicht?
    if ( (Display_breite - Ball_breite) > x )
    {
      x = x + 1;
    }
    else
    {
      richtung = LINKS;
    }
  }

  // Ball fliegt nach links
  if ( richtung == LINKS )
  {
    // Wurde der linke Rand erreicht?
    if ( x > 0 )
    {
      x = x - 1;
    }
    else
    {
      richtung = RECHTS;
    }
  }

  return x;
}
// **********************************************
// Das Zeichnen des Ball's mit den Regeln - ENDE
// **********************************************


// **********************************************
// Abfragen der Köpfe - START
// **********************************************
int Joystick_abfragen_rauf_runter (int y)
{
  // **********************************************
  // Wurde der Knopf "rauf" gedrückt?
  // **********************************************
  if (arduboy.pressed(UP_BUTTON))
  {
    // Ja der Knopf "rauf" wurde gedrückt, Regel anwenden
    y = y - 1;
  }

  // **********************************************
  // Wurde der Knopf "runter" gedrückt?
  // **********************************************
  if (arduboy.pressed(DOWN_BUTTON))
  {
    // Ja der Knopf "runter" wurde gedrückt, Regel anwenden
    y = y + 1;
  }

  return y;
}
// **********************************************
// Abfragen der Köpfe - ENDE
// **********************************************


// **********************************************
// Abfragen der Köpfe (rechts, links) - START
// **********************************************
int Joystick_abfragen_rechts_links (int x)
{
  // **********************************************
  // Wurde der Knopf "links" gedrückt?
  // **********************************************
  if (arduboy.pressed(LEFT_BUTTON))
  {
    // Ja der Knopf "links" wurde gedrückt, Regel anwenden
    x = 0;
  }

  // **********************************************
  // Wurde der Knopf "rechts" gedrückt?
  // **********************************************
  if (arduboy.pressed(RIGHT_BUTTON))
  {
    // Ja der Knopf "rechts" wurde gedrückt, Regel anwenden
    x = Display_breite - 1;
  }

  return x;
}
// **********************************************
// Abfragen der Köpfe (rechts, links) - ENDE
// **********************************************


// **********************************************
// Saegezahn - START
// **********************************************
int Saegezahn (int y)
{
  if (richtung_saegezahn == RUNTER)
  {
    if ( (Display_hoehe - Ball_hoehe) > y )
    {
      y = y + 1;
    }
    else
    {
      richtung_saegezahn = RAUF;
    }
  }

  if (richtung_saegezahn == RAUF)
  {
    if ( y > 0 )
    {
      y = y - 1;
    }
    else
    {
      richtung_saegezahn = RUNTER;
    }
  }

  return y;
}
// **********************************************
// Wellenform - ENDE
// **********************************************


// **********************************************
// Kollissionsdetektierung - START
// **********************************************
int Kollissionsdetektierung(int bx, int by, int sy, int sx, int sl)
{
  int ergebnis = KEINE_KONTROLLE;

  by = by + 8;
  
  if ( (bx == 0) && (sx == 0) )
  {
    if ( (by > sy) && ( by < (sy + sl)) )
    {
      ergebnis = KOLLISION;
    }
    else
    {
      ergebnis = KEINE_KOLLISION;
    }
  }

  if ( (bx == (Display_breite - Ball_breite)) && (sx == (Display_breite - 1)) )
  {
    if ( (by > sy) && ( by < (sy + sl)) )
    {
      ergebnis = KOLLISION;
    }
    else
    {
      ergebnis = KEINE_KOLLISION;
    }
  }

  return ergebnis;
}
// **********************************************
// Kollissionsdetektierung - Ende
// **********************************************
